#Bug.py

import math
import pygame

from engine.AtlanSprite import AtlanSprite

from common.Utility import *

import random

from Vector import Vec2

class Bug(AtlanSprite):
    """
        Annoying pest who likes to eat plants.
        Has basically two states:  hungry and not hungry.
        When hungry, look for the nearest plant and start eating.
        When not hungry, wander around the map.

        The AI is built upon Craig Reynold's rules for steering
        (see www.shiffman.net/teaching/nature/steering for a good intro).
    """


    def __init__(self, myBounds):
        """
            Create the bug at at the top left corner of the provided rectangle.
        """
        AtlanSprite.__init__(self)
        self.bounds = myBounds
        self.trueX = self.bounds.x / 1.0
        self.trueY = self.bounds.y / 1.0
        self.velocity = Vec2(0.0, 0.0)
        self.acceleration = Vec2(0.0, 0.0)
        self.wanderRadius = 30.0
        self.wanderDistance = 10.0
        self.wanderAngle = 0.25
        self.maxSpeed = 1.0
        self.maxForce = 0.5
        self.confidence = 0
        self.target = None
        self.tTarget = None
        self.resetConfidence()

    def steer(self, target):
        """
            Return a vector corresponding to how the current velocity should
            change to steer towards a target location vector.
        """
        desired = Vec2(target.x - self.trueX, target.y - self.trueY)
        d = desired.magnitude()
        steerV = Vec2(0.0, 0.0)
        desired.normalize()
        desired.multiply(self.maxSpeed)

        steerV.x = desired.x - self.velocity.x
        steerV.y = desired.y - self.velocity.y
        steerV.limit(self.maxForce)
        return steerV

    def wander(self):
        """
            Accelerate in a wandering direction (pseudorandomly)
        """
        self.wanderAngle += (random.random() - 0.5) / 2.0

        cLoc = Vec2(self.velocity.x, self.velocity.y)
        cLoc.normalize()
        cLoc.multiply(self.wanderDistance)
        cLoc.x += self.trueX
        cLoc.y += self.trueY

        cOffset = Vec2(self.wanderRadius * math.cos(self.wanderAngle), self.wanderRadius * math.sin(self.wanderAngle))
        target = Vec2(cLoc.x + cOffset.x, cLoc.y + cOffset.y)

        self.acceleration.add(self.steer(target))

    


    def draw(self, surface):
        """
            Draw as a small circle (radius 2).  Red if mad, black if not mad.
        """
        if self.confidence <= 0 and self.target != None:
            pygame.draw.circle(surface, (100, 0, 0, 255), (self.bounds.x, self.bounds.y), 2)
        else:
            pygame.draw.circle(surface, (0, 0, 0, 255), (self.bounds.x, self.bounds.y), 2)

    def resetConfidence(self):
        """
            Make this fly not hungry.  It is assigned a random value to wait until it gets hungry again.
        """
        if self.confidence <= 0:
            self.confidence = random.randint(10000, 30000)

    def update(self, delta):
        """
            Update the logic of the fly by the given timestep.
        """
        self.confidence -= delta
        if self.confidence <= 0 and self.target != None:
            #steer towards target
            self.acceleration.add(self.steer(self.target))
        else:
            #if near the y edges then make sure we steer upwards
            if self.trueY < 15:
                self.velocity.y += 1.0
                self.wanderAngle = 1.57
            elif self.trueY > YRES - 15:
                self.velocity.y += -1.0
                self.wanderAngle = 4.71
            #wander randomly
            self.wander()

        #handle acceleration, velocity, position
        self.velocity.add(self.acceleration)
        self.velocity.limit(self.maxSpeed)
        self.trueX += self.velocity.x
        self.trueY += self.velocity.y

        self.bounds.x = round(self.trueX)
        self.bounds.y = round(self.trueY)
        self.acceleration.multiply(0)

        #wrap on the x direction of the screen
        if self.bounds.x >= XRES + 5:
            self.trueX = -4
        elif self.bounds.x <= -5:
            self.trueX = XRES - 4

        
            
